# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class ShareServersAdminTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ShareServersAdminTest, cls).skip_checks()
        if not CONF.share.multitenancy_enabled:
            raise cls.skipException(
                'Share servers can be tested only with multitenant drivers.')

    @classmethod
    def resource_setup(cls):
        super(ShareServersAdminTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share in this new share network
        cls.share = cls.create_share(
            share_type_id=cls.share_type_id)
        cls.share_network = cls.shares_v2_client.get_share_network(
            cls.shares_v2_client.share_network_id)['share_network']

        if not cls.share_network["name"]:
            sn_id = cls.share_network["id"]
            cls.share_network = cls.shares_v2_client.update_share_network(
                sn_id, name="sn_%s" % sn_id)

        cls.sn_name_and_id = [
            cls.share_network["name"],
            cls.share_network["id"],
        ]
        cls.share_net_info = (
            utils.share_network_get_default_subnet(cls.share_network)
            if utils.share_network_subnets_are_supported()
            else cls.share_network)

        # Date should be like '2014-13-12T11:10:09.000000'
        cls.date_re = re.compile("^([0-9]{4}-[0-9]{2}-[0-9]{2}[A-Z]{1}"
                                 "[0-9]{2}:[0-9]{2}:[0-9]{2}).*$")

    @decorators.idempotent_id('3f821248-2c05-4323-a95f-a0216a537b0a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_servers_without_filters(self):
        servers = self.shares_v2_client.list_share_servers()['share_servers']
        self.assertGreater(len(servers), 0)
        keys = [
            "id",
            "host",
            "status",
            "share_network_name",
            "updated_at",
            "project_id",
        ]

        for server in servers:
            # All expected keys are present
            for key in keys:
                self.assertIn(key, server.keys())
            # 'Updated at' is valid date if set
            if server["updated_at"]:
                self.assertTrue(self.date_re.match(server["updated_at"]))
            # Host is not empty
            self.assertGreater(len(server["host"]), 0)
            # Id is not empty
            self.assertGreater(len(server["id"]), 0)

        # Do not verify statuses because we get all share servers from whole
        # cluster and here can be servers with any state.
        # Server we used is present.
        any(s["share_network_name"] in self.sn_name_and_id for s in servers)

    @decorators.idempotent_id('515e3332-cce9-4a28-ad2e-96dbb2cc672e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_servers_with_host_filter(self):
        # Get list of share servers and remember 'host' name
        servers = self.shares_v2_client.list_share_servers()['share_servers']
        # Remember name of server that was used by this test suite
        # to be sure it will be still existing.
        for server in servers:
            if server["share_network_name"] in self.sn_name_and_id:
                if not server["host"]:
                    msg = ("Server '%s' has wrong value for host - "
                           "'%s'.") % (server["id"], server["host"])
                    raise lib_exc.InvalidContentType(message=msg)
                host = server["host"]
                break
        else:
            msg = ("Appropriate server was not found. Its share_network_data"
                   ": '%s'. List of servers: '%s'.") % (self.sn_name_and_id,
                                                        str(servers))
            raise lib_exc.NotFound(message=msg)
        search_opts = {"host": host}
        servers = self.shares_v2_client.list_share_servers(
            search_opts)['share_servers']
        self.assertGreater(len(servers), 0)
        for server in servers:
            self.assertEqual(server["host"], host)

    @decorators.idempotent_id('77a5c1c5-db2f-4ff6-872c-ed73a575acd0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_servers_with_status_filter(self):
        search_opts = {"status": "active"}
        servers = self.shares_v2_client.list_share_servers(
            search_opts)['share_servers']

        # At least 1 share server should exist always - the one created
        # for this class.
        self.assertGreater(len(servers), 0)
        for server in servers:
            self.assertEqual(server["status"], "active")

    @decorators.idempotent_id('ed671cbb-7732-4966-9f1e-b997b8aa05ff')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_servers_with_project_id_filter(self):
        search_opts = {"project_id": self.share_network["project_id"]}
        servers = self.shares_v2_client.list_share_servers(
            search_opts)['share_servers']
        # Should exist, at least, one share server, used by this test suite.
        self.assertGreater(len(servers), 0)
        for server in servers:
            self.assertEqual(server["project_id"],
                             self.share_network["project_id"])

    @decorators.idempotent_id('70064129-df97-4dba-9482-2819558111ec')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_servers_with_share_network_name_filter(self):
        search_opts = {"share_network": self.share_network["name"]}
        servers = self.shares_v2_client.list_share_servers(
            search_opts)['share_servers']
        # Should exist, at least, one share server, used by this test suite.
        self.assertGreater(len(servers), 0)
        for server in servers:
            self.assertEqual(server["share_network_name"],
                             self.share_network["name"])

    @decorators.idempotent_id('f64e8024-eb42-4a4c-be6b-97d35633a019')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_servers_with_share_network_id_filter(self):
        search_opts = {"share_network": self.share_network["id"]}
        servers = self.shares_v2_client.list_share_servers(
            search_opts)['share_servers']
        # Should exist, at least, one share server, used by this test suite.
        self.assertGreater(len(servers), 0)
        for server in servers:
            self.assertIn(server["share_network_name"],
                          self.sn_name_and_id)

    @decorators.idempotent_id('e1af24f4-bf63-467d-a857-3a402fa9b65b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_show_share_server(self):
        share = self.shares_v2_client.get_share(self.share["id"])['share']
        server = self.shares_v2_client.show_share_server(
            share["share_server_id"])['share_server']
        keys = [
            "id",
            "host",
            "project_id",
            "status",
            "share_network_name",
            "created_at",
            "updated_at",
            "backend_details",
        ]
        if utils.is_microversion_ge(CONF.share.max_api_microversion, "2.49"):
            keys.append("is_auto_deletable")
            keys.append("identifier")
        if utils.is_microversion_ge(
            CONF.share.max_api_microversion, "2.51") and (
                utils.is_microversion_lt(
                    CONF.share.max_api_microversion, "2.70")):
            keys.append("share_network_subnet_id")
        if utils.is_microversion_ge(CONF.share.max_api_microversion, "2.70"):
            keys.append("share_network_subnet_ids")
        # all expected keys are present
        for key in keys:
            self.assertIn(key, server.keys())

        # 'created_at' is valid date
        self.assertTrue(self.date_re.match(server["created_at"]))

        # 'updated_at' is valid date if set
        if server["updated_at"]:
            self.assertTrue(self.date_re.match(server["updated_at"]))

        # veriy that values for following keys are not empty
        for k in ('host', 'id', 'status'):
            self.assertGreater(len(server[k]), 0)

        # 'backend_details' should be a dict
        self.assertIsInstance(server["backend_details"], dict)

    @decorators.idempotent_id('782d8f5f-2c02-44dd-8d43-e06b651a71be')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_show_share_server_details(self):
        share = self.shares_v2_client.get_share(self.share['id'])['share']
        details = self.shares_v2_client.show_share_server_details(
            share['share_server_id'])['details']

        # If details are present they and their values should be only strings
        for k, v in details.items():
            self.assertIsInstance(k, str)
            self.assertIsInstance(v, str)

    @decorators.idempotent_id('2fdf8d29-3ab8-4424-b684-6253f45b9666')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(True, False)
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    def test_delete_share_server(self, delete_share_network):
        # Get network and subnet from existing share_network and reuse it
        # to be able to delete share_server after test ends.
        # TODO(vponomaryov): attach security-services too. If any exist from
        #                    donor share-network.
        new_sn = self.create_share_network(
            add_security_services=True,
            neutron_net_id=self.share_net_info['neutron_net_id'],
            neutron_subnet_id=self.share_net_info['neutron_subnet_id'])

        # Create server with share
        self.create_share(share_type_id=self.share_type_id,
                          share_network_id=new_sn['id'])

        # List share servers, filtered by share_network_id
        servers = self.shares_v2_client.list_share_servers(
            {"share_network": new_sn["id"]})['share_servers']

        # There can be more than one share server for share network when retry
        # was used and share was created successfully not from first time.
        # So, iterate all share-servers, release all created resources. It will
        # allow share network to be deleted in cleanup.
        for serv in servers:
            # Verify that filtering worked as expected.
            self.assertEqual(new_sn["id"], serv["share_network_id"])

            # List shares by share server id
            shares = self.shares_v2_client.list_shares_with_detail(
                {"share_server_id": serv["id"]})['shares']
            for s in shares:
                self.assertEqual(new_sn["id"], s["share_network_id"])

            # Delete shares, so we will have share server without shares
            for s in shares:
                self.shares_v2_client.delete_share(s["id"])

            # Wait for shares deletion
            for s in shares:
                self.shares_v2_client.wait_for_resource_deletion(
                    share_id=s["id"])

            # List shares by share server id, we expect empty list
            empty = self.shares_v2_client.list_shares_with_detail(
                {"share_server_id": serv["id"]})['shares']
            self.assertEqual(0, len(empty))

            if delete_share_network:
                # Delete share network, it should trigger share server deletion
                self.shares_v2_client.delete_share_network(new_sn["id"])
            else:
                # Delete share server
                self.shares_v2_client.delete_share_server(serv["id"])

            # Wait for share server deletion
            self.shares_v2_client.wait_for_resource_deletion(
                server_id=serv["id"])

            if delete_share_network:
                self.shares_v2_client.wait_for_resource_deletion(
                    sn_id=new_sn["id"])

    @decorators.idempotent_id('960d49c5-f8d6-42be-aaf4-b7890a55df03')
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.49")
    def test_share_server_reset_state(self):
        # Get network and subnet from existing share_network and reuse it
        # to be able to delete share_server after test ends.
        new_sn = self.create_share_network(
            add_security_services=True,
            neutron_net_id=self.share_net_info['neutron_net_id'],
            neutron_subnet_id=self.share_net_info['neutron_subnet_id'])
        share = self.create_share(
            share_type_id=self.share_type_id,
            share_network_id=new_sn['id']
        )
        share = self.shares_v2_client.get_share(share['id'])['share']

        # obtain share server
        share_server = self.shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']

        for state in (constants.SERVER_STATE_ACTIVE,
                      constants.SERVER_STATE_CREATING,
                      constants.SERVER_STATE_DELETING,
                      constants.SERVER_STATE_ERROR,
                      constants.SERVER_STATE_MANAGE_ERROR,
                      constants.SERVER_STATE_MANAGE_STARTING,
                      constants.SERVER_STATE_UNMANAGE_ERROR,
                      constants.SERVER_STATE_UNMANAGE_STARTING):

            # leave it in a new state
            self.shares_v2_client.share_server_reset_state(
                share_server['id'],
                status=state,
            )
            waiters.wait_for_resource_status(
                self.shares_v2_client, share_server['id'], state,
                resource_name="share_server"
            )

        # bring the share server back in the active state
        self.shares_v2_client.share_server_reset_state(
            share_server['id'],
            status=constants.SERVER_STATE_ACTIVE,
        )
        waiters.wait_for_resource_status(
            self.shares_v2_client, share_server['id'],
            constants.SERVER_STATE_ACTIVE, resource_name="share_server"
        )

        # delete share
        self.shares_v2_client.delete_share(share["id"])
        self.shares_v2_client.wait_for_resource_deletion(
            share_id=share["id"]
        )

        # delete share network. This will trigger share server deletion
        self.shares_v2_client.delete_share_network(new_sn["id"])
        self.shares_v2_client.wait_for_resource_deletion(
            sn_id=new_sn['id'])



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions


class ShareBuildErrorException(exceptions.TempestException):
    message = "Share %(resource_id)s failed to build and is in ERROR status"


class ShareInstanceBuildErrorException(exceptions.TempestException):
    message = ("Share instance %(resource_id)s failed to build and is in "
               "ERROR status")


class ShareGroupBuildErrorException(exceptions.TempestException):
    message = ("Share group %(resource_id)s failed to build and "
               "is in ERROR status")


class AccessRuleBuildErrorException(exceptions.TempestException):
    message = "Share's rule with id %(resource_id)s is in ERROR status"


class SnapshotBuildErrorException(exceptions.TempestException):
    message = "Snapshot %(resource_id)s failed to build and is in ERROR status"


class SnapshotInstanceBuildErrorException(exceptions.TempestException):
    message = ("Snapshot instance %(resource_id)s failed to build and is in "
               "ERROR status.")


class ShareGroupSnapshotBuildErrorException(exceptions.TempestException):
    message = ("Share Group Snapshot %(resource_id)s failed "
               "to build and is in ERROR status")


class ShareProtocolNotSpecified(exceptions.TempestException):
    message = "Share can not be created, share protocol is not specified"


class ShareNetworkNotSpecified(exceptions.TempestException):
    message = "Share can not be created, share network not specified"


class NoAvailableNetwork(exceptions.TempestException):
    message = "No available network for service VM"


class InvalidResource(exceptions.TempestException):
    message = "Provided invalid resource: %(message)s"


class ShareMigrationException(exceptions.TempestException):
    message = ("Share %(share_id)s failed to migrate from "
               "host %(src)s to host %(dest)s.")


class ResourceReleaseFailed(exceptions.TempestException):
    message = "Failed to release resource '%(res_type)s' with id '%(res_id)s'."


class ShareReplicationTypeException(exceptions.TempestException):
    message = ("Option backend_replication_type is set to incorrect value: "
               "%(replication_type)s")


class ShareServerBuildErrorException(exceptions.TempestException):
    message = ("Share server %(server_id)s failed to build and is in ERROR "
               "status")


class ShareServerMigrationException(exceptions.TempestException):
    message = ("Share server %(server_id)s failed to migrate and is in ERROR "
               "status")


class ShareBackupBuildErrorException(exceptions.TempestException):
    message = ("Share backup %(backup_id)s failed and is in ERROR status")



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest import clients
from tempest import config
from tempest.lib import auth
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils

from manila_tempest_tests.common import waiters

CONF = config.CONF


class ShareRbacBaseTests(object):

    identity_version = 'v3'
    protocols = ['nfs', 'cifs', 'glusterfs', 'hdfs', 'cephfs', 'maprfs']

    @classmethod
    def skip_checks(cls):
        super(ShareRbacBaseTests, cls).skip_checks()
        if not CONF.enforce_scope.manila:
            raise cls.skipException(
                "Tempest is not configured to enforce_scope for manila, "
                "skipping RBAC tests. To enable these tests set "
                "`tempest.conf [enforce_scope] manila=True`."
            )
        if not CONF.share.default_share_type_name:
            raise cls.skipException("Secure rbac tests require a default "
                                    "share type")
        if not any(p in CONF.share.enable_protocols for p in cls.protocols):
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

    @classmethod
    def delete_resource(cls, client, **kwargs):
        key_names = {
            'st': 'share_type',
            'sn': 'share_network',
        }
        key, resource_id = list(kwargs.items())[0]
        key = key.rsplit('_', 1)[0]
        resource_name = key_names[key] if key in key_names else key

        del_action = getattr(client, 'delete_{}'.format(resource_name))
        test_utils.call_and_ignore_notfound_exc(
            del_action, resource_id)
        test_utils.call_and_ignore_notfound_exc(
            client.wait_for_resource_deletion, **kwargs)

    @classmethod
    def create_share(cls, client, share_type_id, size=None, name=None,
                     metadata=None):
        kwargs = {}
        name = name or data_utils.rand_name('share')
        metadata = metadata or {}
        kwargs.update({
            'share_protocol': cls.protocol,
            'size': size or CONF.share.share_size,
            'name': name or data_utils.rand_name('share'),
            'share_type_id': share_type_id,
            'metadata': metadata,
        })
        share = client.create_share(**kwargs)['share']
        waiters.wait_for_resource_status(client, share['id'], 'available')
        cls.addClassResourceCleanup(
            cls.delete_resource, client,
            share_id=share['id'])
        return share

    @classmethod
    def create_snapshot(cls, client, share_id, name=None, metadata=None):
        name = name or data_utils.rand_name('snapshot')
        snapshot = client.create_snapshot(
            share_id, name=name, metadata=metadata)['snapshot']
        waiters.wait_for_resource_status(
            client, snapshot['id'], 'available', resource_name='snapshot')
        cls.addClassResourceCleanup(
            cls.delete_resource, client, snapshot_id=snapshot['id'])
        return snapshot

    @classmethod
    def create_share_network(cls, client, name=None):
        name = name or data_utils.rand_name('share_network')
        share_network = client.create_share_network(name=name)['share_network']

        cls.addClassResourceCleanup(
            cls.delete_resource, client, sn_id=share_network['id'])
        return share_network

    @classmethod
    def create_share_type(cls):
        name = data_utils.rand_name('share-type')
        extra_specs = {
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        share_type = cls.admin_shares_v2_client.create_share_type(
            name=name, extra_specs=extra_specs)['share_type']
        cls.addClassResourceCleanup(
            cls.delete_resource, cls.admin_shares_v2_client,
            st_id=share_type['id'])
        return share_type

    @classmethod
    def create_share_group_type(cls, share_types, is_public=True,
                                group_specs=None):
        name = data_utils.rand_name('share-group-type')
        share_group_type = (
            cls.admin_shares_v2_client.create_share_group_type(
                name=name, share_types=share_types, is_public=is_public,
                group_specs=group_specs))['share_group_type']
        cls.addClassResourceCleanup(
            cls.delete_resource, cls.admin_shares_v2_client,
            share_group_type_id=share_group_type['id'])
        return share_group_type

    @classmethod
    def create_share_group(cls, client, share_group_type_id, share_type_ids):
        name = data_utils.rand_name('share-group')
        share_group = client.create_share_group(
            name=name, share_group_type_id=share_group_type_id,
            share_type_ids=share_type_ids)['share_group']
        waiters.wait_for_resource_status(
            client, share_group['id'], 'available',
            resource_name='share_group')
        cls.addClassResourceCleanup(
            cls.delete_resource, client,
            share_group_id=share_group['id'])
        return share_group

    @classmethod
    def get_share_type(cls):
        return cls.shares_v2_client.get_default_share_type()['share_type']

    def do_request(self, method, expected_status=200, client=None, **payload):
        if not client:
            client = self.client
        if isinstance(expected_status, type(Exception)):
            self.assertRaises(expected_status,
                              getattr(client, method),
                              **payload)
        else:
            response = getattr(client, method)(**payload)
            self.assertEqual(response.response.status, expected_status)
            return response

    @classmethod
    def setup_user_client(cls, client, project_id=None):
        """Set up project user with its own client.

        This is useful for testing protection of resources in separate
        projects.
        NOTE(lkuchlan): Tempest creates 'project_member' and 'project_reader'
        dynamic credentials in different projects. So this method is also
        necessary for testing protection of resources in a specific project.

        Returns a client object and the user's ID.
        """

        projects_client = client.identity_v3.ProjectsClient()
        users_client = client.identity_v3.UsersClient()
        roles_client = client.identity_v3.RolesClient()

        user_dict = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password(),
        }
        user_id = users_client.create_user(
            **user_dict)['user']['id']
        cls.addClassResourceCleanup(users_client.delete_user, user_id)

        if not project_id:
            project_id = projects_client.create_project(
                data_utils.rand_name())['project']['id']
            cls.addClassResourceCleanup(
                projects_client.delete_project,
                project_id)

        member_role_id = roles_client.list_roles(
            name='member')['roles'][0]['id']
        roles_client.create_user_role_on_project(
            project_id, user_id, member_role_id)
        creds = auth.KeystoneV3Credentials(
            user_id=user_id,
            password=user_dict['password'],
            project_id=project_id)
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        client = clients.Manager(credentials=creds)
        return client



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareGroupTypesAdminNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareGroupTypesAdminNegativeTest, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ShareGroupTypesAdminNegativeTest, cls).resource_setup()
        cls.share_type = cls.create_share_type(
            data_utils.rand_name("unique_st_name"),
            extra_specs=cls.add_extra_specs_to_dict({"key": "value"}),
            client=cls.admin_shares_v2_client)
        cls.share_group_type = cls.create_share_group_type(
            data_utils.rand_name("unique_sgt_name"),
            share_types=[cls.share_type['id']],
            group_specs={"key": "value"},
            client=cls.admin_shares_v2_client)

    @decorators.idempotent_id('1f8e3f98-4df7-4383-94d6-4ad058ef79c1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type_without_name(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.create_share_group_type,
            name=None,
            share_types=data_utils.rand_name("fake"))

    @decorators.idempotent_id('6c95f5b7-6e30-4e1e-b0fc-a0f05cf2982d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type_with_nonexistent_share_type(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.create_share_group_type,
            name=data_utils.rand_name("sgt_name_should_have_not_been_created"),
            share_types=data_utils.rand_name("fake"))

    @decorators.idempotent_id('ce6ba41b-4207-4866-8f70-7a89f64b4cd4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type_with_empty_name(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group_type, '',
            client=self.admin_shares_v2_client)

    @decorators.idempotent_id('77159ec6-adb7-4d35-9d4b-039d322f7852')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type_with_too_big_name(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group_type,
            "x" * 256, client=self.admin_shares_v2_client)

    @decorators.idempotent_id('43ac388d-05e8-45d5-b633-631cc1f290af')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type_with_wrong_value_for_group_specs(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.create_share_group_type,
            name=data_utils.rand_name("tempest_manila"),
            share_types=[self.share_type['id']],
            group_specs="expecting_error_code_400")

    @decorators.idempotent_id('8fb8bd73-0219-460d-993e-bff7ddec29e8')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_share_group_type_using_nonexistent_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_group_type,
            data_utils.rand_name("fake"))

    @decorators.idempotent_id('76b5c302-f492-45b4-b464-7be6efd373be')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_share_group_type_using_nonexistent_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.delete_share_group_type,
            data_utils.rand_name("fake"))

    @decorators.idempotent_id('081183cb-0f1c-4b67-8552-909935dc0be2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_duplicate_of_share_group_type(self):
        unique_name = data_utils.rand_name("unique_sgt_name")
        list_of_ids = set()
        for _ in (1, 2):
            sg_type = self.create_share_group_type(
                unique_name,
                share_types=[self.share_type['id']],
                client=self.admin_shares_v2_client,
                cleanup_in_class=False)
            self.assertRaises(
                lib_exc.Conflict,
                self.create_share_group_type,
                unique_name,
                share_types=[self.share_type['id']],
                client=self.admin_shares_v2_client)
            list_of_ids.add(sg_type['id'])
            self.assertEqual(unique_name, sg_type['name'])
            self.admin_shares_v2_client.delete_share_group_type(sg_type['id'])
        self.assertEqual(2, len(list_of_ids))

    @decorators.idempotent_id('6c30989d-2e8a-48d3-809a-556dddae51bd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_project_access_to_public_share_group_type(self):
        self.assertRaises(
            lib_exc.Conflict,
            self.admin_shares_v2_client.add_access_to_share_group_type,
            self.share_group_type["id"],
            self.admin_shares_v2_client.tenant_id)

    @decorators.idempotent_id('3375a1f8-7828-4cbe-89d7-fe11fcf30b21')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_remove_project_access_from_public_share_group_type(self):
        self.assertRaises(
            lib_exc.Conflict,
            self.admin_shares_v2_client.remove_access_from_share_group_type,
            self.share_group_type["id"],
            self.admin_shares_v2_client.tenant_id)

    @decorators.idempotent_id('68e3d7f9-fdc0-44a2-ae84-a72eba314e19')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_project_access_to_nonexistent_share_group_type(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.add_access_to_share_group_type,
            data_utils.rand_name("fake"),
            self.admin_shares_v2_client.tenant_id)

    @decorators.idempotent_id('94f0c902-ab12-435e-abd5-e09a892db82a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_remove_project_access_from_nonexistent_share_group_type(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.remove_access_from_share_group_type,
            data_utils.rand_name("fake"),
            self.admin_shares_v2_client.tenant_id)

    @decorators.idempotent_id('3e763f5b-6663-4620-9471-ed3050da6201')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_share_group_type_extra_specs_with_user(self):
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.get_share_group_type_specs,
            self.share_group_type['id'])

    @decorators.idempotent_id('2264f7eb-3ff0-47e9-8ab0-54694113db3d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_extra_specs_from_nonexistent_share_group_type(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_group_type_specs,
            data_utils.rand_name('fake'))

    @decorators.idempotent_id('2be79455-0ce7-4ca6-818f-40651ba79c6e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_extra_spec_with_nonexistent_key(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_group_type_spec,
            self.share_group_type['id'], 'fake_key')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from oslo_log import log
from tempest import config
from tempest.lib.common import ssh
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions

CONF = config.CONF

LOG = log.getLogger(__name__)


def debug_ssh(function):
    """Decorator to generate extra debug info in case of ssh failure"""
    def wrapper(self, *args, **kwargs):
        try:
            return function(self, *args, **kwargs)
        except exceptions.SSHTimeout:
            try:
                original_exception = sys.exc_info()
                caller = test_utils.find_test_caller() or "not found"
                if self.server:
                    msg = 'Caller: %s. Timeout trying to ssh to server %s'
                    LOG.debug(msg, caller, self.server)
                    if self.log_console and self.servers_client:
                        try:
                            msg = 'Console log for server %s: %s'
                            console_log = (
                                self.servers_client.get_console_output(
                                    self.server['id'])['output'])
                            LOG.debug(msg, self.server['id'], console_log)
                        except Exception:
                            msg = 'Could not get console_log for server %s'
                            LOG.debug(msg, self.server['id'])
                # re-raise the original ssh timeout exception
                raise original_exception
            finally:
                # Delete the traceback to avoid circular references
                _, _, trace = original_exception
                del trace
    return wrapper


class RemoteClient(object):

    def __init__(self, ip_address, username, password=None, pkey=None,
                 server=None, servers_client=None):
        """Executes commands in a VM over ssh

        :param ip_address: IP address to ssh to
        :param username: ssh username
        :param password: ssh password (optional)
        :param pkey: ssh public key (optional)
        :param server: server dict, used for debugging purposes
        :param servers_client: servers client, used for debugging purposes
        """
        self.server = server
        self.servers_client = servers_client
        self.log_console = CONF.compute_feature_enabled.console_output
        kwargs = {}

        try:
            kwargs['ssh_key_type'] = CONF.validation.ssh_key_type
        except Exception:
            # Not all versions of tempest support the
            # "validation.ssh_key_type" config option
            pass

        self.ssh_client = ssh.Client(
            ip_address, username, password, pkey=pkey, **kwargs)

    @debug_ssh
    def exec_command(self, cmd):
        # Shell options below add more clearness on failures,
        # path is extended for some non-cirros guest oses (centos7)
        cmd = CONF.validation.ssh_shell_prologue + " " + cmd
        LOG.debug("Remote command: %s", cmd)
        return self.ssh_client.exec_command(cmd)

    @debug_ssh
    def validate_authentication(self):
        """Validate ssh connection and authentication

           This method raises an Exception when the validation fails.
        """
        self.ssh_client.test_connection_auth()



# Copyright 2016 Yogesh Kshirsagar
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
_MIN_SUPPORTED_MICROVERSION = '2.11'


class ReplicationSnapshotTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ReplicationSnapshotTest, cls).skip_checks()
        if not CONF.share.run_replication_tests:
            raise cls.skipException('Replication tests are disabled.')
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            _MIN_SUPPORTED_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ReplicationSnapshotTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.replication_type = CONF.share.backend_replication_type
        cls.multitenancy_enabled = (
            utils.replication_with_multitenancy_support())

        if cls.replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=cls.replication_type
            )

        # create share type
        extra_specs = {
            "replication_type": cls.replication_type,
            "snapshot_support": True,
        }
        if CONF.share.capability_create_share_from_snapshot_support:
            extra_specs.update({
                "create_share_from_snapshot_support": True,
            })
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        cls.sn_id = None
        if cls.multitenancy_enabled:
            cls.share_network = cls.shares_v2_client.get_share_network(
                cls.shares_v2_client.share_network_id)['share_network']
            cls.sn_id = cls.share_network['id']

        cls.zones = cls.get_availability_zones_matching_share_type(
            cls.share_type, client=cls.admin_client)
        cls.share_zone = cls.zones[0]
        cls.replica_zone = cls.zones[-1]

    @decorators.idempotent_id('67fce02a-1d46-4186-8260-69e30b26195d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_snapshot_after_share_replica(self):
        """Test the snapshot for replicated share.

        Create replica first and then create a snapshot.
        Verify that the snapshot is properly created under replica by
        creating a share from that snapshot.
        """
        share = self.create_share(share_type_id=self.share_type_id,
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        original_replica = self.shares_v2_client.list_share_replicas(
            share["id"])['share_replicas'][0]

        share_replica = self.create_share_replica(share["id"],
                                                  self.replica_zone,
                                                  cleanup=False)
        self.addCleanup(self.delete_share_replica, original_replica['id'])
        waiters.wait_for_resource_status(
            self.shares_v2_client, share_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')

        snapshot = self.create_snapshot_wait_for_active(share["id"])
        self.promote_share_replica(share_replica['id'])
        self.delete_share_replica(original_replica['id'])

        snapshot = self.shares_v2_client.get_snapshot(
            snapshot['id'])['snapshot']
        self.assertEqual(constants.STATUS_AVAILABLE, snapshot['status'])

        if CONF.share.capability_create_share_from_snapshot_support:
            self.create_share(share_type_id=self.share_type_id,
                              snapshot_id=snapshot['id'],
                              share_network_id=self.sn_id)

    @decorators.idempotent_id('ced8920b-6e3a-4bb4-9b3d-86e54377fcb7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_snapshot_before_share_replica(self):
        """Test the snapshot for replicated share.

        Create snapshot before creating share replica for the same
        share.
        Verify snapshot by creating share from the snapshot.
        """
        share = self.create_share(share_type_id=self.share_type_id,
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        snapshot = self.create_snapshot_wait_for_active(share["id"])

        original_replica = self.shares_v2_client.list_share_replicas(
            share["id"])['share_replicas'][0]
        share_replica = self.create_share_replica(share["id"],
                                                  self.replica_zone,
                                                  cleanup=False)
        self.addCleanup(self.delete_share_replica, original_replica['id'])
        waiters.wait_for_resource_status(
            self.shares_v2_client, share_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')

        # Wait for snapshot1 to become available
        waiters.wait_for_resource_status(
            self.shares_v2_client, snapshot['id'], "available",
            resource_name='snapshot')

        self.promote_share_replica(share_replica['id'])
        self.delete_share_replica(original_replica['id'])

        snapshot = self.shares_v2_client.get_snapshot(
            snapshot['id'])['snapshot']
        self.assertEqual(constants.STATUS_AVAILABLE, snapshot['status'])

        if CONF.share.capability_create_share_from_snapshot_support:
            self.create_share(share_type_id=self.share_type_id,
                              snapshot_id=snapshot['id'],
                              share_network_id=self.sn_id)

    @decorators.idempotent_id('a5372844-69ad-40e1-bbf3-265eb66af123')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_snapshot_before_and_after_share_replica(self):
        """Test the snapshot for replicated share.

        Verify that snapshot can be created before and after share replica
        being created.
        Verify snapshots by creating share from the snapshots.
        """
        share = self.create_share(share_type_id=self.share_type_id,
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        snapshot1 = self.create_snapshot_wait_for_active(share["id"])

        original_replica = self.shares_v2_client.list_share_replicas(
            share["id"])['share_replicas'][0]

        share_replica = self.create_share_replica(share["id"],
                                                  self.replica_zone,
                                                  cleanup=False)
        self.addCleanup(self.delete_share_replica, original_replica['id'])
        waiters.wait_for_resource_status(
            self.shares_v2_client, share_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')

        snapshot2 = self.create_snapshot_wait_for_active(share["id"])

        # Wait for snapshot1 to become available
        waiters.wait_for_resource_status(
            self.shares_v2_client, snapshot1['id'], "available",
            resource_name='snapshot')

        self.promote_share_replica(share_replica['id'])
        # Remove the original active replica to ensure that snapshot is
        # still being created successfully.
        self.delete_share_replica(original_replica['id'])

        snapshot1 = self.shares_v2_client.get_snapshot(
            snapshot1['id'])['snapshot']
        self.assertEqual(constants.STATUS_AVAILABLE, snapshot1['status'])

        snapshot2 = self.shares_v2_client.get_snapshot(
            snapshot2['id'])['snapshot']
        self.assertEqual(constants.STATUS_AVAILABLE, snapshot2['status'])

        if CONF.share.capability_create_share_from_snapshot_support:
            self.create_share(share_type_id=self.share_type_id,
                              snapshot_id=snapshot1['id'],
                              share_network_id=self.sn_id)
            self.create_share(share_type_id=self.share_type_id,
                              snapshot_id=snapshot2['id'],
                              share_network_id=self.sn_id)

    @decorators.idempotent_id('b4cd376b-1eb8-4780-801a-64b0c3bee1e4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_delete_snapshot_after_adding_replica(self):
        """Verify the snapshot delete.

        Ensure that deleting the original snapshot also deletes the
        snapshot from replica.
        """

        share = self.create_share(share_type_id=self.share_type_id,
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        share_replica = self.create_share_replica(share["id"],
                                                  self.replica_zone)
        waiters.wait_for_resource_status(
            self.shares_v2_client, share_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')
        snapshot = self.create_snapshot_wait_for_active(share["id"])
        self.shares_v2_client.delete_snapshot(snapshot['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            snapshot_id=snapshot["id"])

    @decorators.idempotent_id('913056a7-d897-4a1b-a7be-b2b2cf5d9572')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Create share from snapshot tests are disabled.")
    def test_create_replica_from_snapshot_share(self):
        """Test replica for a share that was created from snapshot."""

        share = self.create_share(share_type_id=self.share_type_id,
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        orig_snapshot = self.create_snapshot_wait_for_active(share["id"])
        snap_share = self.create_share(share_type_id=self.share_type_id,
                                       snapshot_id=orig_snapshot['id'],
                                       share_network_id=self.sn_id)
        original_replica = self.shares_v2_client.list_share_replicas(
            snap_share["id"])['share_replicas'][0]
        share_replica = self.create_share_replica(snap_share["id"],
                                                  self.replica_zone,
                                                  cleanup=False)
        self.addCleanup(self.delete_share_replica, original_replica['id'])
        waiters.wait_for_resource_status(
            self.shares_v2_client, share_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')
        self.promote_share_replica(share_replica['id'])
        # Delete the demoted replica so promoted replica can be cleaned
        # during the cleanup
        self.delete_share_replica(original_replica['id'])



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareRbacShareGroupTypesTests(rbac_base.ShareRbacBaseTests,
                                    metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacShareGroupTypesTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.admin_shares_v2_client = (
            cls.os_project_admin.share_v2.SharesV2Client())

    @classmethod
    def skip_checks(cls):
        super(ShareRbacShareGroupTypesTests, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ShareRbacShareGroupTypesTests, cls).resource_setup()
        cls.group_specs1 = {'key1': 'value1'}
        cls.group_specs2 = {'key2': 'value2'}
        cls.share_type = cls.create_share_type()
        cls.share_group_type = cls.create_share_group_type(
            cls.share_type['id'], group_specs=cls.group_specs1)
        cls.private_share_group_type = cls.create_share_group_type(
            cls.share_type['id'], is_public=False)

    @abc.abstractmethod
    def test_create_share_group_type(self):
        pass

    @abc.abstractmethod
    def test_get_share_group_type(self):
        pass

    @abc.abstractmethod
    def test_list_share_group_types(self):
        pass

    @abc.abstractmethod
    def test_delete_share_group_type(self):
        pass

    @abc.abstractmethod
    def test_create_share_group_type_extra_specs(self):
        pass

    @abc.abstractmethod
    def test_update_share_group_type_extra_spec(self):
        pass

    @abc.abstractmethod
    def test_delete_share_group_type_extra_spec(self):
        pass

    @abc.abstractmethod
    def test_add_share_group_type_access(self):
        pass

    @abc.abstractmethod
    def test_list_share_group_type_access(self):
        pass

    @abc.abstractmethod
    def test_remove_share_group_type_access(self):
        pass


class ProjectAdminTests(ShareRbacShareGroupTypesTests, base.BaseSharesTest):

    credentials = ['project_admin']

    @decorators.idempotent_id('9ea9954a-ae09-4d02-a082-9a72b80009fc')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_share_group_type(self):
        share_group_type = self.do_request(
            'create_share_group_type', expected_status=200,
            name='gt', share_types=self.share_type['id'])['share_group_type']
        self.addCleanup(self.delete_resource, self.client,
                        share_group_type_id=share_group_type['id'])

    @decorators.idempotent_id('fcad2b86-ca43-42b0-82bd-37e6f760e4d2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_share_group_type(self):
        self.do_request(
            'get_share_group_type', expected_status=200,
            share_group_type_id=self.share_group_type['id'])

    @decorators.idempotent_id('7871b1b5-610a-425c-9363-d0bcf2beff72')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_group_types(self):
        share_group_type_list = self.do_request(
            'list_share_group_types', expected_status=200)['share_group_types']
        share_group_type_id_list = [
            sgt['id'] for sgt in share_group_type_list
        ]
        self.assertIn(self.share_group_type['id'], share_group_type_id_list)

    @decorators.idempotent_id('c23da121-8b1f-4443-80cc-11881745a1c3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_delete_share_group_type(self):
        share_group_type = self.create_share_group_type(
            share_types=self.share_type['id'])
        self.do_request(
            'delete_share_group_type', expected_status=204,
            share_group_type_id=share_group_type['id'])

    @decorators.idempotent_id('80eb22cb-846a-4b51-a71d-ceef0b804901')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_share_group_type_extra_specs(self):
        self.do_request(
            'create_share_group_type_specs', expected_status=200,
            share_group_type_id=self.share_group_type['id'],
            group_specs_dict=self.group_specs2)
        self.addCleanup(
            self.admin_shares_v2_client.delete_share_group_type_spec,
            self.share_group_type['id'], group_spec_key='key2')

    @decorators.idempotent_id('fe29877d-2226-42ca-b492-4be0dacd6eaf')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_share_group_type_extra_spec(self):
        self.do_request(
            'update_share_group_type_spec', expected_status=200,
            share_group_type_id=self.share_group_type['id'],
            group_spec_key='key', group_spec_value='value_updated')

    @decorators.idempotent_id('743c18dc-8c3a-4934-9ef8-8b342daffe7c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_delete_share_group_type_extra_spec(self):
        self.admin_shares_v2_client.create_share_group_type_specs(
            self.share_group_type['id'], self.group_specs2)
        self.do_request(
            'delete_share_group_type_spec', expected_status=204,
            share_group_type_id=self.share_group_type['id'],
            group_spec_key='key2')

    @decorators.idempotent_id('89876c46-1167-450d-8b98-746d97fff388')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_add_share_group_type_access(self):
        self.do_request(
            'add_access_to_share_group_type', expected_status=202,
            share_group_type_id=self.private_share_group_type['id'],
            project_id=self.client.project_id)
        self.addCleanup(
            self.client.remove_access_from_share_group_type,
            share_group_type_id=self.private_share_group_type['id'],
            project_id=self.client.project_id)

    @decorators.idempotent_id('64a7be53-d1af-40c3-950b-743a2704ac97')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_group_type_access(self):
        self.client.add_access_to_share_group_type(
            self.private_share_group_type['id'], self.client.project_id)
        self.addCleanup(
            self.client.remove_access_from_share_group_type,
            share_group_type_id=self.private_share_group_type['id'],
            project_id=self.client.project_id)
        access_list = self.do_request(
            'list_access_to_share_group_type', expected_status=200,
            share_group_type_id=self.private_share_group_type['id']
        )['share_group_type_access']

        project_id_list = [
            access['project_id'] for access in access_list
        ]

        self.assertIn(self.client.project_id, project_id_list)

    @decorators.idempotent_id('fc67ba18-78e9-4c02-9b37-2bd49c8a4470')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_remove_share_group_type_access(self):
        self.client.add_access_to_share_group_type(
            self.private_share_group_type['id'], self.client.project_id)
        self.do_request(
            'remove_access_from_share_group_type', expected_status=202,
            share_group_type_id=self.private_share_group_type['id'],
            project_id=self.client.project_id)


class ProjectMemberTests(ShareRbacShareGroupTypesTests, base.BaseSharesTest):

    credentials = ['project_member', 'project_admin']

    @decorators.idempotent_id('f9e6f2fd-7c1a-4eee-817c-bf1988904515')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type(self):
        self.do_request(
            'create_share_group_type', expected_status=lib_exc.Forbidden,
            name='gt', share_types=self.share_type['id'])

    @decorators.idempotent_id('8eaf4a99-9706-41c9-8b12-40856d0900f4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_share_group_type(self):
        self.do_request(
            'get_share_group_type', expected_status=200,
            share_group_type_id=self.share_group_type['id'])

    @decorators.idempotent_id('d2dd61f4-c763-49f9-9c93-8b587879f554')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_group_types(self):
        share_group_type_list = self.do_request(
            'list_share_group_types', expected_status=200)['share_group_types']
        share_group_type_id_list = [
            sgt['id'] for sgt in share_group_type_list
        ]
        self.assertIn(self.share_group_type['id'], share_group_type_id_list)

    @decorators.idempotent_id('8f45798f-717d-41b0-acba-a80dd647cddf')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_group_type(self):
        share_group_type = self.create_share_group_type(
            share_types=self.share_type['id'])
        self.do_request(
            'delete_share_group_type', expected_status=lib_exc.Forbidden,
            share_group_type_id=share_group_type['id'])

    @decorators.idempotent_id('2fa2f953-f068-4c60-ab52-eeb1bd69a7a4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type_extra_specs(self):
        self.do_request(
            'create_share_group_type_specs', expected_status=lib_exc.Forbidden,
            share_group_type_id=self.share_group_type['id'],
            group_specs_dict=self.group_specs2)

    @decorators.idempotent_id('30c73c94-b7bc-4e1f-8172-192335997879')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_share_group_type_extra_spec(self):
        self.do_request(
            'update_share_group_type_spec', expected_status=lib_exc.Forbidden,
            share_group_type_id=self.share_group_type['id'],
            group_spec_key='key', group_spec_value='value_updated')

    @decorators.idempotent_id('95aa1fbf-3eaf-4d65-96b5-4554b0fb0937')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_group_type_extra_spec(self):
        self.admin_shares_v2_client.create_share_group_type_specs(
            self.share_group_type['id'], self.group_specs2)
        self.do_request(
            'delete_share_group_type_spec', expected_status=lib_exc.Forbidden,
            share_group_type_id=self.share_group_type['id'],
            group_spec_key='key2')

    @decorators.idempotent_id('85b69e26-9c67-45c4-80e0-2ce212977c2a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_share_group_type_access(self):
        self.do_request(
            'add_access_to_share_group_type',
            expected_status=lib_exc.Forbidden,
            share_group_type_id=self.private_share_group_type['id'],
            project_id=self.client.project_id)

    @decorators.idempotent_id('027314a9-6b14-4ae9-83f2-471e84ccaa01')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_share_group_type_access(self):
        self.admin_shares_v2_client.add_access_to_share_group_type(
            self.private_share_group_type['id'], self.client.project_id)
        self.addCleanup(
            self.admin_shares_v2_client.remove_access_from_share_group_type,
            share_group_type_id=self.private_share_group_type['id'],
            project_id=self.client.project_id)
        self.do_request(
            'list_access_to_share_group_type',
            expected_status=lib_exc.Forbidden,
            share_group_type_id=self.private_share_group_type['id'])

    @decorators.idempotent_id('d8518122-dabd-4d2d-8b6a-4eed975e19ee')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_remove_share_group_type_access(self):
        self.admin_shares_v2_client.add_access_to_share_group_type(
            self.private_share_group_type['id'], self.client.project_id)
        self.addCleanup(
            self.admin_shares_v2_client.remove_access_from_share_group_type,
            share_group_type_id=self.private_share_group_type['id'],
            project_id=self.client.project_id)
        self.do_request(
            'remove_access_from_share_group_type',
            expected_status=lib_exc.Forbidden,
            share_group_type_id=self.private_share_group_type['id'],
            project_id=self.client.project_id)


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'project_member', 'project_admin']

    @decorators.idempotent_id('8c47bbe9-f3f1-419e-b00b-97c9c942a48a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type(self):
        super(ProjectReaderTests, self).test_create_share_group_type()

    @decorators.idempotent_id('fe3b28a3-6980-4782-8eaa-518bbd3913d1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_share_group_type(self):
        super(ProjectReaderTests, self).test_get_share_group_type()

    @decorators.idempotent_id('47f92f0b-424e-4685-a742-8a4e00cc6901')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_group_types(self):
        super(ProjectReaderTests, self).test_list_share_group_types()

    @decorators.idempotent_id('e853fd60-8906-4e38-b0b4-ec5723696518')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_group_type(self):
        super(ProjectReaderTests, self).test_delete_share_group_type()

    @decorators.idempotent_id('caa6d960-4f34-4dff-9cc0-9a0cde44c2ae')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_group_type_extra_specs(self):
        super(
            ProjectReaderTests,
            self).test_create_share_group_type_extra_specs()

    @decorators.idempotent_id('2782c329-2447-49f7-95e7-0c4c766cfda3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_share_group_type_extra_spec(self):
        super(
            ProjectReaderTests, self).test_update_share_group_type_extra_spec()

    @decorators.idempotent_id('003a0eee-0075-47b0-ba8a-8e57e958c1d3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_group_type_extra_spec(self):
        super(
            ProjectReaderTests, self).test_delete_share_group_type_extra_spec()

    @decorators.idempotent_id('93bba264-104c-48af-af91-5a79ec3e695e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_share_group_type_access(self):
        super(ProjectReaderTests, self).test_add_share_group_type_access()

    @decorators.idempotent_id('176b280c-7a46-43ae-8164-3bdd945d2dd4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_share_group_type_access(self):
        super(ProjectReaderTests, self).test_list_share_group_type_access()

    @decorators.idempotent_id('00caa020-3805-4322-aac1-86d4552268a2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_remove_share_group_type_access(self):
        super(ProjectReaderTests, self).test_remove_share_group_type_access()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from oslo_utils import timeutils
from oslo_utils import uuidutils
from tempest import config
from tempest.lib import decorators

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF

MICROVERSION = '2.37'
QUERY_BY_TIMESTAMP_MICROVERSION = '2.52'
MESSAGE_KEYS = (
    'created_at',
    'action_id',
    'detail_id',
    'expires_at',
    'id',
    'message_level',
    'request_id',
    'resource_type',
    'resource_id',
    'user_message',
    'project_id',
    'links',
)


class UserMessageTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(UserMessageTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported(MICROVERSION)

    def setUp(self):
        super(UserMessageTest, self).setUp()
        self.message = self.create_user_message()

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API])
    @decorators.idempotent_id('37127e11-7aa7-46b2-ab05-e3bf36d94fd8')
    def test_list_messages(self):
        body = self.shares_v2_client.list_messages()['messages']
        self.assertIsInstance(body, list)
        self.assertTrue(self.message['id'], [x['id'] for x in body])
        message = body[0]
        self.assertEqual(set(MESSAGE_KEYS), set(message.keys()))

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API])
    @decorators.idempotent_id('ff938d3e-9cdd-470a-b091-d389be7be755')
    def test_list_messages_sorted_and_paginated(self):
        self.create_user_message()
        self.create_user_message()
        params = {'sort_key': 'resource_id', 'sort_dir': 'asc', 'limit': 2}
        body = self.shares_v2_client.list_messages(params=params)
        # tempest/lib/common/rest_client.py's _parse_resp checks
        # for number of keys in response's dict, if there is only single
        # key, it returns directly this key, otherwise it returns
        # parsed body. If limit param is used, then API returns
        # multiple keys in response ('messages' and 'message_links')
        messages = body['messages']
        self.assertIsInstance(messages, list)
        ids = [x['resource_id'] for x in messages]
        self.assertEqual(2, len(ids))
        self.assertEqual(ids, sorted(ids))

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API])
    @decorators.idempotent_id('292b0a2c-f066-48a6-9492-8dede87ee7b4')
    def test_list_messages_filtered(self):
        self.create_user_message()
        params = {'resource_id': self.message['resource_id']}
        body = self.shares_v2_client.list_messages(params=params)['messages']
        self.assertIsInstance(body, list)
        ids = [x['id'] for x in body]
        self.assertEqual([self.message['id']], ids)

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API])
    @decorators.idempotent_id('a92971c5-a72a-4de2-8e9b-dc612b798a7a')
    def test_show_message(self):
        self.addCleanup(self.shares_v2_client.delete_message,
                        self.message['id'])

        message = self.shares_v2_client.get_message(
            self.message['id'])['message']

        self.assertEqual(set(MESSAGE_KEYS), set(message.keys()))
        self.assertTrue(uuidutils.is_uuid_like(message['id']))
        self.assertEqual('001', message['action_id'])
        # don't check specific detail_id which may vary
        # depending on order of filters, we can still check
        # user_message
        self.assertIn(
            'No storage could be allocated for this share request',
            message['user_message'])
        self.assertEqual('SHARE', message['resource_type'])
        self.assertTrue(uuidutils.is_uuid_like(message['resource_id']))
        self.assertEqual('ERROR', message['message_level'])
        created_at = timeutils.parse_strtime(message['created_at'])
        expires_at = timeutils.parse_strtime(message['expires_at'])
        self.assertGreater(expires_at, created_at)
        self.assertEqual(set(MESSAGE_KEYS), set(message.keys()))

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API])
    @decorators.idempotent_id('88b2db98-8e33-4552-9225-c19a3b6f18e0')
    def test_delete_message(self):
        self.shares_v2_client.delete_message(self.message['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            message_id=self.message['id'])

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API])
    @utils.skip_if_microversion_not_supported(QUERY_BY_TIMESTAMP_MICROVERSION)
    @decorators.idempotent_id('2ed0c40e-cdaa-471b-97d4-5ebe3fb040e9')
    def test_list_messages_with_since_and_before_filters(self):
        new_message = self.create_user_message()
        created_at_1 = timeutils.parse_strtime(self.message['created_at'])
        created_at_2 = timeutils.parse_strtime(new_message['created_at'])
        time_1 = created_at_1 - datetime.timedelta(seconds=1)
        time_2 = created_at_2 - datetime.timedelta(seconds=1)

        params1 = {'created_since': str(time_1)}
        # should return all user messages created by this test including
        # self.message
        messages = self.shares_v2_client.list_messages(
            params=params1)['messages']
        ids = [x['id'] for x in messages]
        self.assertGreaterEqual(len(ids), 2)
        self.assertIn(self.message['id'], ids)
        self.assertIn(new_message['id'], ids)
        for message in messages:
            time_diff_with_created_since = timeutils.delta_seconds(
                time_1, timeutils.parse_strtime(message['created_at']))
            self.assertGreaterEqual(time_diff_with_created_since, 0)

        params2 = {'created_since': str(time_1),
                   'created_before': str(time_2)}
        # should not return new_message, but return a list that is equal to 1
        # and include self.message
        messages = self.shares_v2_client.list_messages(
            params=params2)['messages']
        self.assertIsInstance(messages, list)
        ids = [x['id'] for x in messages]
        self.assertGreaterEqual(len(ids), 1)
        self.assertIn(self.message['id'], ids)
        self.assertNotIn(new_message['id'], ids)
        for message in messages:
            time_diff_with_created_since = timeutils.delta_seconds(
                time_1, timeutils.parse_strtime(message['created_at']))
            time_diff_with_created_before = timeutils.delta_seconds(
                time_2, timeutils.parse_strtime(message['created_at']))
            self.assertGreaterEqual(time_diff_with_created_since, 0)
            self.assertGreaterEqual(0, time_diff_with_created_before)

        params3 = {'created_before': str(time_2)}
        # should not include self.message
        messages = self.shares_v2_client.list_messages(
            params=params3)['messages']
        ids = [x['id'] for x in messages]
        self.assertGreaterEqual(len(ids), 1)
        self.assertNotIn(new_message['id'], ids)
        self.assertIn(self.message['id'], ids)
        for message in messages:
            time_diff_with_created_before = timeutils.delta_seconds(
                time_2, timeutils.parse_strtime(message['created_at']))
            self.assertGreaterEqual(0, time_diff_with_created_before)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacShareNetworkTests(rbac_base.ShareRbacBaseTests,
                                 metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacShareNetworkTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.alt_project_share_v2_client = (
            cls.os_project_alt_member.share_v2.SharesV2Client())

    @classmethod
    def resource_setup(cls):
        super(ShareRbacShareNetworkTests, cls).resource_setup()
        cls.share_type = cls.get_share_type()

    @abc.abstractmethod
    def test_create_share_network(self):
        pass

    @abc.abstractmethod
    def test_list_share_network(self):
        pass

    @abc.abstractmethod
    def test_show_share_network(self):
        pass

    @abc.abstractmethod
    def test_delete_share_network(self):
        pass

    @abc.abstractmethod
    def test_update_share_network(self):
        pass


class ProjectAdminTests(ShareRbacShareNetworkTests, base.BaseSharesTest):

    credentials = ['project_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super(ProjectAdminTests, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.persona, project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('358dd850-cd81-4b81-aefa-3dfcb7aa4551')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_share_network(self):
        share_network = self.do_request(
            'create_share_network', expected_status=200)['share_network']
        self.addCleanup(
            self.delete_resource, self.client, sn_id=share_network['id'])

    @decorators.idempotent_id('deb20301-9d7c-4c08-b1f0-fc2c403ea708')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_network(self):
        share_network = self.create_share_network(self.share_member_client)
        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        params = {"all_tenants": 1}
        share_network_list = self.do_request(
            'list_share_networks', expected_status=200,
            params=params)['share_networks']
        share_network_id_list = [
            s['id'] for s in share_network_list
        ]

        self.assertIn(share_network['id'], share_network_id_list)
        self.assertIn(alt_share_network['id'], share_network_id_list)

    @decorators.idempotent_id('43a3be84-d08b-4f17-89cf-02abda6df580')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_show_share_network(self):
        share_network = self.create_share_network(self.share_member_client)
        self.do_request(
            'get_share_network', expected_status=200,
            share_network_id=share_network['id'])

        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        self.do_request(
            'get_share_network', expected_status=200,
            share_network_id=alt_share_network['id'])

    @decorators.idempotent_id('6c403ed6-b810-4794-8e9b-d57f173443a2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_delete_share_network(self):
        share_network = self.create_share_network(self.share_member_client)
        self.do_request(
            'delete_share_network', expected_status=202,
            sn_id=share_network['id'])

        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        self.do_request(
            'delete_share_network', expected_status=202,
            sn_id=alt_share_network['id'])

    @decorators.idempotent_id('abd2443d-3490-462a-8e51-73b6a8f48795')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_share_network(self):
        share_network = self.create_share_network(self.share_member_client)
        name = data_utils.rand_name("updated_share_network")
        self.do_request(
            'update_share_network', expected_status=200,
            sn_id=share_network['id'], name=name)

        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        self.do_request(
            'update_share_network', expected_status=200,
            sn_id=alt_share_network['id'], name=name)


class ProjectMemberTests(ShareRbacShareNetworkTests, base.BaseSharesTest):

    credentials = ['project_member', 'project_alt_member']

    @decorators.idempotent_id('d051c749-3d1c-4485-86c5-6eb860b49cad')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_share_network(self):
        share_network = self.do_request(
            'create_share_network', expected_status=200)['share_network']
        self.addCleanup(
            self.delete_resource, self.client, sn_id=share_network['id'])

    @decorators.idempotent_id('ac33cd51-1efe-4aaf-99ab-b510b7551571')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_network(self):
        share_client = getattr(self, 'share_member_client', self.client)
        share_network = self.create_share_network(share_client)
        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        share_network_list = self.do_request(
            'list_share_networks', expected_status=200)['share_networks']
        share_network_id_list = [
            s['id'] for s in share_network_list
        ]

        self.assertIn(share_network['id'], share_network_id_list)
        self.assertNotIn(alt_share_network['id'], share_network_id_list)

    @decorators.idempotent_id('dc3f8f95-f8c5-4030-93dd-e4c56e40b477')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_show_share_network(self):
        share_client = getattr(self, 'share_member_client', self.client)
        share_network = self.create_share_network(share_client)
        self.do_request(
            'get_share_network', expected_status=200,
            share_network_id=share_network['id'])

        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        self.do_request(
            'get_share_network', expected_status=lib_exc.NotFound,
            share_network_id=alt_share_network['id'])

    @decorators.idempotent_id('717977ab-f077-411a-9bdc-06c8ec9d4f8c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_network(self):
        share_network = self.create_share_network(self.client)
        self.do_request(
            'delete_share_network', expected_status=202,
            sn_id=share_network['id'])

        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        self.do_request(
            'delete_share_network', expected_status=lib_exc.NotFound,
            sn_id=alt_share_network['id'])

    @decorators.idempotent_id('d1fce94c-b163-452d-bf79-13b6edf47e30')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_share_network(self):
        share_network = self.create_share_network(self.client)
        name = data_utils.rand_name("updated_share_network")
        self.do_request(
            'update_share_network', expected_status=200,
            sn_id=share_network['id'], name=name)

        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        self.do_request(
            'update_share_network', expected_status=lib_exc.NotFound,
            sn_id=alt_share_network['id'], name=name)


class ProjectReaderTests(ProjectMemberTests):
    """Test suite for basic share network operations by reader user

    In order to test certain share operations we must create a share network
    resource for this. Since reader user is limited in resources creation, we
    are forced to use admin credentials, so we can test other share
    operations. In this class we use admin user to create a member user within
    reader project. That way we can perform a reader actions on this resource.
    """

    credentials = ['project_reader', 'project_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super(ProjectReaderTests, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.os_project_admin,
            project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()
        cls.alt_project_share_v2_client = (
            cls.os_project_alt_member.share_v2.SharesV2Client())

    @decorators.idempotent_id('73dd9f09-7106-4fd5-a484-0eb986002e3b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_network(self):
        self.do_request(
            'create_share_network', expected_status=lib_exc.Forbidden)

    @decorators.idempotent_id('841e9e69-2a22-4572-9147-b233c8a842bc')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_network(self):
        super(ProjectReaderTests, self).test_list_share_network()

    @decorators.idempotent_id('c98893c8-cdc6-42af-a842-1ee9466904ae')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_show_share_network(self):
        super(ProjectReaderTests, self).test_show_share_network()

    @decorators.idempotent_id('f8f26bce-ff82-4472-a8dd-0f46c1757386')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_network(self):
        share_network = self.create_share_network(self.share_member_client)
        self.do_request(
            'delete_share_network', expected_status=lib_exc.Forbidden,
            sn_id=share_network['id'])

        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        self.do_request(
            'delete_share_network', expected_status=lib_exc.Forbidden,
            sn_id=alt_share_network['id'])

    @decorators.idempotent_id('67b745cd-e669-4872-bbb7-9307960fbd77')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_share_network(self):
        share_network = self.create_share_network(self.share_member_client)
        name = data_utils.rand_name("updated_share_network")
        self.do_request(
            'update_share_network', expected_status=lib_exc.Forbidden,
            sn_id=share_network['id'], name=name)

        alt_share_network = self.create_share_network(
            self.alt_project_share_v2_client)
        self.do_request(
            'update_share_network', expected_status=lib_exc.Forbidden,
            sn_id=alt_share_network['id'], name=name)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import time

import ddt
from oslo_log import log as logging
from oslo_utils import units
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.scenario import manager_share as manager


CONF = config.CONF
LOG = logging.getLogger(__name__)


@ddt.ddt
class ShareExtendBase(manager.ShareScenarioTest):

    """This test case uses the following flow:

     * Launch an instance
     * Create share (1GB)
     * Configure RW access to the share
     * Perform ssh to instance
     * Mount share
     * Write data in share
     * Extend share (2GB)
     * Write more data in share
     * Unmount share
     * Delete share
     * Terminate the instance
    """

    @decorators.idempotent_id('e1c0d614-c8f2-43cf-9c49-25808b07ba4a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_extend_and_write(self):
        default_share_size = CONF.share.share_size
        additional_overflow_blocks = CONF.share.additional_overflow_blocks
        share_growth_size = CONF.share.share_growth_size

        LOG.debug('Step 1 - create instance')
        instance = self.boot_instance(wait_until="BUILD")

        LOG.debug('Step 2 - create share of size {} Gb'
                  .format(default_share_size))
        share = self.create_share(size=default_share_size)

        LOG.debug('Step 3 - wait for active instance')
        instance = self.wait_for_active_instance(instance["id"])
        remote_client = self.init_remote_client(instance)

        LOG.debug('Step 4 - grant access')
        location = self.get_user_export_locations(share)[0]
        self.allow_access(share=share, instance=instance,
                          remote_client=remote_client, locations=location)

        LOG.debug('Step 5 - mount')
        self.mount_share(location, remote_client)

        total_blocks = (units.Ki * default_share_size) / 64
        three_quarter_blocks = (total_blocks / 4) * 3
        LOG.debug('Step 6 - writing {} * 64MB blocks'
                  .format(three_quarter_blocks))
        self.write_data_to_mounted_share_using_dd(remote_client,
                                                  '/mnt/t1', '64M',
                                                  three_quarter_blocks)
        time.sleep(CONF.share.share_resize_sync_delay)
        ls_result = remote_client.exec_command("sudo ls -lAh /mnt/")
        LOG.debug(ls_result)

        # Additional blocks that exceed the share capacity, defaulting to 5.
        overflow_blocks = (
            total_blocks - three_quarter_blocks +
            additional_overflow_blocks or 5)
        LOG.debug('Step 6b - Write more data, should fail')
        self.assertRaises(
            exceptions.SSHExecCommandFailed,
            self.write_data_to_mounted_share_using_dd,
            remote_client, '/mnt/t2', '64M', overflow_blocks)
        time.sleep(CONF.share.share_resize_sync_delay)
        ls_result = remote_client.exec_command("sudo ls -lAh /mnt/")
        LOG.debug(ls_result)

        LOG.debug('Step 7 - extend and wait')
        extended_share_size = default_share_size + share_growth_size
        self.shares_v2_client.extend_share(share["id"],
                                           new_size=extended_share_size)
        waiters.wait_for_resource_status(
            self.shares_v2_client, share["id"], constants.STATUS_AVAILABLE)
        share = self.shares_v2_client.get_share(share["id"])['share']
        self.assertEqual(extended_share_size, int(share["size"]))
        time.sleep(CONF.share.share_resize_sync_delay)

        LOG.debug('Step 8 - writing more data, should succeed')
        self.write_data_with_remount(location, remote_client, '/mnt/t3',
                                     '64M', overflow_blocks)
        ls_result = remote_client.exec_command("sudo ls -lAh /mnt/")
        LOG.debug(ls_result)

        LOG.debug('Step 9 - unmount')
        self.unmount_share(remote_client)

    def write_data_with_remount(self, mount_location,
                                remote_client,
                                output_file,
                                block_size,
                                block_count):
        """Writes data to mounted share using dd command

        Tries remounting once if encountering a stale file handle

        :param mount_location: Mount point for remounting if needed
        :param remote_client: An SSH client connection to the Nova instance
        :param block_size: The size of an individual block in bytes
        :param block_count: The number of blocks to write
        :param output_file: Path to the file to be written
        """

        try:
            self.write_data_to_mounted_share_using_dd(remote_client,
                                                      output_file,
                                                      block_size,
                                                      block_count)
        except exceptions.SSHExecCommandFailed as e:
            if 'stale file handle' in str(e).lower():
                LOG.warning("Client was disconnected during extend process")
                self.unmount_share(remote_client)
                self.mount_share(mount_location, remote_client)
                self.write_data_to_mounted_share_using_dd(remote_client,
                                                          output_file,
                                                          block_size,
                                                          block_count)
            else:
                raise


class TestShareExtendNFS(manager.BaseShareScenarioNFSTest, ShareExtendBase):
    pass


class TestShareExtendCIFS(manager.BaseShareScenarioCIFSTest, ShareExtendBase):

    @decorators.idempotent_id('4a9f5cf9-990d-4a35-9ac7-87ef593a09e3')
    @decorators.unstable_test(bug='1903922')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_extend_and_write(self):
        super(TestShareExtendCIFS, self).test_create_extend_and_write()


class TestBaseShareExtendScenarioCEPHFS(manager.BaseShareScenarioCEPHFSTest,
                                        ShareExtendBase):

    @decorators.idempotent_id('9ca1e4a9-23e3-4da6-a63e-46e7919335e0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_extend_and_write_with_ceph_fuse_client(self):
        self.mount_client = 'fuse'
        super(TestBaseShareExtendScenarioCEPHFS,
              self).test_create_extend_and_write()


class TestShareExtendNFSIPv6(TestShareExtendNFS):
    ip_version = 6


# NOTE(u_glide): this function is required to exclude ShareExtendBase
# from executed test cases.
# See: https://docs.python.org/3/library/unittest.html#load-tests-protocol
# for details.
def load_tests(loader, tests, _):
    result = []
    for test_case in tests:
        if type(test_case._tests[0]) is ShareExtendBase:
            continue
        result.append(test_case)
    return loader.suiteClass(result)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

import ddt
from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF
PRE_SHARE_REPLICAS_MICROVERSION = "2.52"
SHARE_REPLICAS_MICROVERSION = "2.53"


@ddt.ddt
class SharesQuotasTest(base.BaseSharesTest):

    @classmethod
    def skip_checks(cls):
        super(SharesQuotasTest, cls).skip_checks()
        if not CONF.share.run_quota_tests:
            msg = "Quota tests are disabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(SharesQuotasTest, cls).resource_setup()
        cls.user_id = cls.shares_v2_client.user_id
        cls.tenant_id = cls.shares_v2_client.tenant_id

    @decorators.idempotent_id('a4649f11-41d0-4e73-ab5a-1466f2339b2f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_default_quotas(self, client_name):
        quotas = getattr(self, client_name).default_quotas(
            self.tenant_id)['quota_set']
        uses_v2_client = client_name == 'shares_v2_client'
        self.assertGreater(int(quotas["gigabytes"]), -2)
        self.assertGreater(int(quotas["snapshot_gigabytes"]), -2)
        self.assertGreater(int(quotas["shares"]), -2)
        self.assertGreater(int(quotas["snapshots"]), -2)
        self.assertGreater(int(quotas["share_networks"]), -2)
        if utils.share_replica_quotas_are_supported() and uses_v2_client:
            self.assertGreater(int(quotas["share_replicas"]), -2)
            self.assertGreater(int(quotas["replica_gigabytes"]), -2)

    @decorators.idempotent_id('19525293-2b32-48c3-a7cd-cab279ac4631')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_show_quotas(self, client_name):
        quotas = getattr(self, client_name).show_quotas(
            self.tenant_id)['quota_set']
        uses_v2_client = client_name == 'shares_v2_client'
        self.assertGreater(int(quotas["gigabytes"]), -2)
        self.assertGreater(int(quotas["snapshot_gigabytes"]), -2)
        self.assertGreater(int(quotas["shares"]), -2)
        self.assertGreater(int(quotas["snapshots"]), -2)
        self.assertGreater(int(quotas["share_networks"]), -2)
        if utils.share_replica_quotas_are_supported() and uses_v2_client:
            self.assertGreater(int(quotas["share_replicas"]), -2)
            self.assertGreater(int(quotas["replica_gigabytes"]), -2)

    @decorators.idempotent_id('1e670af7-9734-42a8-97b7-3080526f3aca')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_show_quotas_for_user(self, client_name):
        quotas = getattr(self, client_name).show_quotas(
            self.tenant_id, self.user_id)['quota_set']
        uses_v2_client = client_name == 'shares_v2_client'
        self.assertGreater(int(quotas["gigabytes"]), -2)
        self.assertGreater(int(quotas["snapshot_gigabytes"]), -2)
        self.assertGreater(int(quotas["shares"]), -2)
        self.assertGreater(int(quotas["snapshots"]), -2)
        self.assertGreater(int(quotas["share_networks"]), -2)
        if utils.share_replica_quotas_are_supported() and uses_v2_client:
            self.assertGreater(int(quotas["share_replicas"]), -2)
            self.assertGreater(int(quotas["replica_gigabytes"]), -2)

    @ddt.data(
        *itertools.product(utils.deduplicate(
            ["2.25", "2.53", CONF.share.max_api_microversion]), (True, False))
    )
    @ddt.unpack
    @decorators.idempotent_id('795614f6-4a18-47d5-b817-0b294e9d4c48')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_show_quotas_detail(self, microversion, with_user):
        utils.check_skip_if_microversion_not_supported(microversion)
        quota_args = {"tenant_id": self.tenant_id, "version": microversion, }
        keys = ['gigabytes', 'snapshot_gigabytes', 'shares',
                'snapshots', 'share_networks']
        if utils.is_microversion_ge(microversion, SHARE_REPLICAS_MICROVERSION):
            keys.append('share_replicas')
            keys.append('replica_gigabytes')
        if with_user:
            quota_args.update({"user_id": self.user_id})
        quotas = self.shares_v2_client.detail_quotas(**quota_args)['quota_set']
        quota_keys = list(quotas.keys())
        for outer in keys:
            self.assertIn(outer, quota_keys)
            outer_keys = list(quotas[outer].keys())
            for inner in ('in_use', 'limit', 'reserved'):
                self.assertIn(inner, outer_keys)
                self.assertGreater(int(quotas[outer][inner]), -2)

    @decorators.idempotent_id('7bd5ac42-9fcb-477f-a253-02cde2bde661')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(PRE_SHARE_REPLICAS_MICROVERSION)
    def test_quota_detail_2_52_no_share_replica_quotas(self):
        quota_args = {"tenant_id": self.tenant_id,
                      "version": PRE_SHARE_REPLICAS_MICROVERSION}
        quotas = self.shares_v2_client.detail_quotas(**quota_args)
        self.assertNotIn('share_replicas', quotas.keys())
        self.assertNotIn('replica_gigabytes', quotas.keys())



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class SharesNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(SharesNegativeTest, cls).resource_setup()
        # create share_type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        if CONF.share.capability_create_share_from_snapshot_support:
            extra_specs.update({'create_share_from_snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

    @decorators.idempotent_id('b9bb8dee-0c7c-4e51-909c-028335b1a6a0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_try_delete_share_with_existing_snapshot(self):
        # share can not be deleted while snapshot exists

        # create share
        share = self.create_share(share_type_id=self.share_type_id)

        # create snapshot
        self.create_snapshot_wait_for_active(share["id"])

        # try delete share
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.delete_share, share["id"])

    @decorators.idempotent_id('3df8e2d8-9b79-428d-9d8b-30bc66b5b40e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Create share from snapshot tests are disabled.")
    def test_create_share_from_snap_with_less_size(self):
        # requires minimum 5Gb available space
        skip_msg = "Check disc space for this test"

        try:  # create share
            size = CONF.share.share_size + 1
            share = self.create_share(size=size,
                                      share_type_id=self.share_type_id,
                                      cleanup_in_class=False)
        except share_exceptions.ShareBuildErrorException:
            self.skip(skip_msg)

        try:  # create snapshot
            snap = self.create_snapshot_wait_for_active(
                share["id"], cleanup_in_class=False)
        except share_exceptions.SnapshotBuildErrorException:
            self.skip(skip_msg)

        # try create share from snapshot with less size
        self.assertRaises(lib_exc.BadRequest,
                          self.create_share,
                          share_type_id=self.share_type_id,
                          snapshot_id=snap["id"],
                          cleanup_in_class=False)

    @decorators.idempotent_id('3047fb1c-5acc-4ef2-8796-b2d2d49829b5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(not CONF.share.multitenancy_enabled,
                      "Only for multitenancy.")
    def test_create_share_with_nonexistant_share_network(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.create_share,
                          share_type_id=self.share_type_id,
                          share_network_id="wrong_sn_id")

    @decorators.idempotent_id('e84ce567-a090-47c7-87c4-6ee427bdee7a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(not CONF.share.multitenancy_enabled,
                      "Only for multitenancy.")
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Create share from snapshot tests are disabled.")
    def test_create_share_from_snap_with_different_share_network(self):
        # We can't create a share from a snapshot whose base share does not
        # have 'create_share_from_snapshot_support'.

        # create share
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # get parent's share network
        parent_share = self.shares_client.get_share(share["id"])['share']
        parent_sn = self.shares_client.get_share_network(
            parent_share["share_network_id"])['share_network']

        # create new share-network - net duplicate of parent's share
        new_duplicated_sn = self.create_share_network(
            cleanup_in_class=False,
            neutron_net_id=parent_sn["neutron_net_id"],
            neutron_subnet_id=parent_sn["neutron_subnet_id"],
        )

        # create snapshot of parent share
        snap = self.create_snapshot_wait_for_active(
            share["id"], cleanup_in_class=False)

        # try create share with snapshot using another share-network
        # 400 bad request is expected
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share,
            share_type_id=self.share_type_id,
            cleanup_in_class=False,
            share_network_id=new_duplicated_sn["id"],
            snapshot_id=snap["id"],
        )


class SharesAPIOnlyNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(SharesAPIOnlyNegativeTest, cls).resource_setup()
        # create share_type
        cls.share_type = cls.create_share_type()
        cls.share_type_min_2_max_5 = cls.create_share_type(
            extra_specs={
                'provisioning:max_share_size': int(CONF.share.share_size) + 4,
                'provisioning:min_share_size': int(CONF.share.share_size) + 1
            })
        cls.share_type_id = cls.share_type['id']
        cls.share_type_min_2_max_5_id = cls.share_type_min_2_max_5['id']

    @decorators.idempotent_id('75837f93-8c2c-40a4-bb9e-d76c53db07c7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_unmanage_share_by_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.unmanage_share,
                          'fake-id')

    @decorators.idempotent_id('97a4dd2f-7c90-4eb7-bf74-d698c3060833')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_manage_share_by_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.manage_share,
                          'fake-host', 'nfs', '/export/path',
                          'fake-type')

    @decorators.idempotent_id('1a438374-8a91-4566-9cef-0386f6609445')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_by_user_with_host_filter(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.list_shares,
                          params={'host': 'fake_host'})

    @decorators.idempotent_id('73f37f33-946d-4213-9d27-25cd4e9a0208')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_by_share_server_by_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.list_shares,
                          params={'share_server_id': 12345})

    @decorators.idempotent_id('2f0df934-b2fb-4ebd-96f7-183cb699dcdd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_non_existent_az(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.create_share,
                          share_type_id=self.share_type_id,
                          availability_zone='fake_az')

    @decorators.idempotent_id('5ae2ecd7-a694-4ba9-ae23-cca75580b9d8')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_with_zero_size(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.create_share,
                          share_type_id=self.share_type_id,
                          size=0)

    @decorators.idempotent_id('3620a380-f9f9-4521-a468-473581637344')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_with_invalid_size(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.create_share,
                          share_type_id=self.share_type_id,
                          size="#$%")

    @decorators.idempotent_id('26ed523d-a215-4661-a038-633b74c9cad7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_with_out_passing_size(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.create_share,
                          share_type_id=self.share_type_id,
                          size="")

    @decorators.idempotent_id('bf303b29-bbcb-4a96-96e9-270e12df58d1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_delete_snapshot_with_wrong_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.delete_snapshot,
                          "wrong_share_id")

    @decorators.idempotent_id('08e5a9c7-45cb-414c-b375-28c335f20ff1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_create_snapshot_with_wrong_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.create_snapshot,
                          "wrong_share_id")

    @decorators.idempotent_id('78e5e327-c68e-4910-82b2-27f5f4d150ac')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_with_invalid_protocol(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.create_share,
                          share_type_id=self.share_type_id,
                          share_protocol="nonexistent_protocol")

    @decorators.idempotent_id('2b7c7ea8-c0e9-446c-a8e3-add765452c04')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_with_wrong_public_value(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.create_share,
                          share_type_id=self.share_type_id,
                          is_public='truebar')

    @decorators.idempotent_id('f82d1667-ae39-43bb-b5aa-bfc9b2ec7292')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_share_with_wrong_id(self):
        self.assertRaises(lib_exc.NotFound, self.shares_client.get_share,
                          "wrong_share_id")

    @decorators.idempotent_id('d03cf44e-6e69-415f-be36-25defb86df56')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_share_without_passing_share_id(self):
        # Should not be able to get share when empty ID is passed
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.get_share, '')

    @decorators.idempotent_id('a9487254-606b-444f-ba6a-2f461bcaf474')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_shares_nonadmin_with_nonexistent_share_server_filter(self):
        # filtering by share server allowed only for admins by default
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.list_shares_with_detail,
                          {'share_server_id': 'fake_share_server_id'})

    @decorators.idempotent_id('9698d1a3-8ee8-46fa-a46b-1084d98e7149')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_with_wrong_id(self):
        self.assertRaises(lib_exc.NotFound, self.shares_client.delete_share,
                          "wrong_share_id")

    @decorators.idempotent_id('b8097d56-067e-4d7c-8401-31bc7021fd24')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_without_passing_share_id(self):
        # Should not be able to delete share when empty ID is passed
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.delete_share, '')

    @utils.skip_if_microversion_not_supported("2.61")
    @decorators.idempotent_id('b8097d56-067e-4d7c-8401-31bc7021fe86')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_size_greater_than_specified_in_share_type(self):
        # Should not be able to create share if size too large
        self.assertRaises(lib_exc.BadRequest,
                          self.create_share,
                          size=int(CONF.share.share_size) + 5,
                          share_type_id=self.share_type_min_2_max_5_id)

    @utils.skip_if_microversion_not_supported("2.61")
    @decorators.idempotent_id('b8097d56-067e-4d7c-8401-31bc7021fe87')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_size_less_than_specified_in_share_type(self):
        # Should not be able to create share if size too small
        self.assertRaises(lib_exc.BadRequest,
                          self.create_share,
                          size=int(CONF.share.share_size),
                          share_type_id=self.share_type_min_2_max_5_id)



# Copyright 2016 Hitachi Data Systems
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.api import test_snapshot_rules
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class SnapshotIpRulesForNFSNegativeTest(
        test_snapshot_rules.BaseShareSnapshotRulesTest):
    protocol = "nfs"

    @classmethod
    def skip_checks(cls):
        super(SnapshotIpRulesForNFSNegativeTest, cls).skip_checks()
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')
        if not CONF.share.run_mount_snapshot_tests:
            raise cls.skipException('Mountable snapshots tests are disabled.')
        if not (cls.protocol in CONF.share.enable_protocols and
                cls.protocol in CONF.share.enable_ip_rules_for_protocols):
            msg = "IP rule tests for %s protocol are disabled." % cls.protocol
            raise cls.skipException(msg)

        utils.check_skip_if_microversion_not_supported('2.32')

    @classmethod
    def resource_setup(cls):
        super(SnapshotIpRulesForNFSNegativeTest, cls).resource_setup()
        # create share type
        extra_specs = {
            'snapshot_support': True,
            'mount_snapshot_support': True,
        }
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)
        cls.snap = cls.create_snapshot_wait_for_active(cls.share["id"])

    @decorators.idempotent_id('a877276e-03d4-4fc5-b4b4-c8874d23f1dc')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data("1.2.3.256", "1.1.1.-", "1.2.3.4/33", "1.2.3.*", "1.2.3.*/23",
              "1.2.3.1|23", "1.2.3.1/", "1.2.3.1/-1",
              "fe80:217:f2ff:fe07:ed62", "2001:db8::1/148",
              "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff",
              "2001:0db8:0000:85a3:0000:0000:ac1f:8001/64")
    def test_create_access_rule_ip_with_wrong_target(self, target):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_snapshot_access_rule,
                          self.snap["id"], "ip", target)

    @decorators.idempotent_id('dd93150f-8081-4533-abb3-f99a03652a87')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data("1.2.3.4", "fd8c:b029:bba6:ac54::1",
              "fd8c:b029:bba6:ac54::1/128", "1.2.3.4/32")
    def test_create_duplicate_of_ip_rule(self, access_to):
        self._test_duplicate_rules(access_to)
        self._test_duplicate_rules(access_to)

    def _test_duplicate_rules(self, access_to):
        if ':' in access_to and utils.is_microversion_lt(
                CONF.share.max_api_microversion, '2.38'):
            reason = ("Skipped. IPv6 rules are accepted from and beyond "
                      "API version 2.38, the configured maximum API version "
                      "is %s" % CONF.share.max_api_microversion)
            raise self.skipException(reason)

        # test data
        access_type = "ip"

        # create rule
        rule = self.shares_v2_client.create_snapshot_access_rule(
            self.snap['id'], access_type, access_to)['snapshot_access']

        waiters.wait_for_resource_status(
            self.shares_v2_client, self.snap['id'], 'active',
            resource_name='snapshot_access', rule_id=rule['id'],
            status_attr='state')

        # try create duplicate of rule
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_snapshot_access_rule,
                          self.snap["id"], access_type, access_to)

        # try alternate notation
        if '/' in access_to:
            access_to = access_to.split("/")[0]
        else:
            access_to = ('%s/32' % access_to if '.' in access_to else
                         '%s/128' % access_to)
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_snapshot_access_rule,
                          self.snap["id"], access_type, access_to)

        # delete rule and wait for deletion
        self.shares_v2_client.delete_snapshot_access_rule(self.snap['id'],
                                                          rule['id'])
        waiters.wait_for_snapshot_access_rule_deletion(
            self.shares_v2_client, self.snap['id'], rule['id'])

        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_snapshot_access_rule,
                          self.snap['id'], rule['id'])



# Copyright 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF


@ddt.ddt
class ManageNFSSnapshotTest(base.BaseSharesAdminTest):
    protocol = 'nfs'

    # NOTE(vponomaryov): be careful running these tests using generic driver
    # because cinder volume snapshots won't be deleted.

    @classmethod
    def skip_checks(cls):
        super(ManageNFSSnapshotTest, cls).skip_checks()
        if not CONF.share.run_manage_unmanage_snapshot_tests:
            raise cls.skipException(
                'Manage/unmanage snapshot tests are disabled.')
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

        utils.check_skip_if_microversion_not_supported('2.12')
        utils.skip_if_manage_not_supported_for_version()

    @classmethod
    def resource_setup(cls):
        super(ManageNFSSnapshotTest, cls).resource_setup()

        # Create share type
        cls.st_name = data_utils.rand_name("tempest-manage-st-name")
        cls.extra_specs = {
            'storage_protocol': CONF.share.capability_storage_protocol,
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
            'snapshot_support': CONF.share.capability_snapshot_support,
        }

        cls.st = cls.create_share_type(
            name=cls.st_name,
            cleanup_in_class=True,
            extra_specs=cls.extra_specs)

        # Create the base share
        cls.share = cls.create_share(share_type_id=cls.st['id'],
                                     share_protocol=cls.protocol)

        # Get updated data
        cls.share = cls.shares_v2_client.get_share(cls.share['id'])['share']

    def _test_manage(self, snapshot, version=CONF.share.max_api_microversion):
        name = ("Name for 'managed' snapshot that had ID %s" %
                snapshot['id'])
        description = "Description for 'managed' snapshot"

        utils.skip_if_manage_not_supported_for_version(version)

        # Manage snapshot
        share_id = snapshot['share_id']
        snapshot = self.shares_v2_client.manage_snapshot(
            share_id,
            snapshot['provider_location'],
            name=name,
            description=description,
            # Some drivers require additional parameters passed as driver
            # options, as follows:
            # - size: Hitachi HNAS Driver
            driver_options={'size': snapshot['size']},
            version=version,
        )['snapshot']

        # Add managed snapshot to cleanup queue
        self.method_resources.insert(
            0, {'type': 'snapshot', 'id': snapshot['id'],
                'client': self.shares_v2_client})

        # Wait for success
        waiters.wait_for_resource_status(
            self.shares_v2_client, snapshot['id'], constants.STATUS_AVAILABLE,
            resource_name='snapshot'
        )

        # Verify manage snapshot API response
        expected_keys = ["status", "links", "share_id", "name",
                         "share_proto", "created_at",
                         "description", "id", "share_size", "size",
                         "provider_location"]
        if utils.is_microversion_ge(version, '2.17'):
            expected_keys.extend(["user_id", "project_id"])
        if utils.is_microversion_ge(version, '2.73'):
            expected_keys.extend(["metadata"])

        actual_keys = snapshot.keys()

        # Strict key check
        self.assertEqual(set(expected_keys), set(actual_keys))

        # Verify data of managed snapshot
        get_snapshot = self.shares_v2_client.get_snapshot(
            snapshot['id'])['snapshot']
        self.assertEqual(name, get_snapshot['name'])
        self.assertEqual(description, get_snapshot['description'])
        self.assertEqual(snapshot['share_id'], get_snapshot['share_id'])

        # Delete snapshot
        self.shares_v2_client.delete_snapshot(get_snapshot['id'])
        self.shares_client.wait_for_resource_deletion(
            snapshot_id=get_snapshot['id'])
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_snapshot,
                          get_snapshot['id'])

    @decorators.idempotent_id('5fb65a19-fb73-4b5a-9210-010f93e0304f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(*utils.deduplicate(['2.12', '2.16', '2.73',
                                  CONF.share.max_api_microversion]))
    def test_manage_different_versions(self, version):
        """Run snapshot manage test for multiple versions.

        This test is configured with ddt to run for the configured maximum
        version as well as versions 2.12 (when the API was introduced) and
        2.16.
        """
        utils.skip_if_manage_not_supported_for_version(version)

        # Skip in case specified version is not supported
        utils.check_skip_if_microversion_not_supported(version)

        snap_name = data_utils.rand_name("tempest-snapshot-name")
        snap_desc = data_utils.rand_name("tempest-snapshot-description")
        # Create snapshot
        snapshot = self.create_snapshot_wait_for_active(
            self.share['id'], snap_name, snap_desc)
        snapshot = self.shares_v2_client.get_snapshot(
            snapshot['id'])['snapshot']
        # Unmanage snapshot
        self.shares_v2_client.unmanage_snapshot(snapshot['id'],
                                                version=version)
        self.shares_client.wait_for_resource_deletion(
            snapshot_id=snapshot['id'])

        # Manage snapshot
        self._test_manage(snapshot=snapshot, version=version)


class ManageCIFSSnapshotTest(ManageNFSSnapshotTest):
    protocol = 'cifs'


class ManageGLUSTERFSSnapshotTest(ManageNFSSnapshotTest):
    protocol = 'glusterfs'


class ManageHDFSSnapshotTest(ManageNFSSnapshotTest):
    protocol = 'hdfs'


class ManageMapRFSSnapshotTest(ManageNFSSnapshotTest):
    protocol = 'maprfs'



# Copyright 2020 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class MigrationShareServerBase(base.BaseSharesAdminTest):
    protocol = None

    @classmethod
    def skip_checks(cls):
        super(MigrationShareServerBase, cls).skip_checks()
        if cls.protocol not in CONF.share.enable_protocols:
            raise cls.skipException('%s tests are disabled.' % cls.protocol)
        if not CONF.share.multitenancy_enabled:
            raise cls.skipException('Multitenancy tests are disabled.')
        if not CONF.share.run_share_server_migration_tests:
            raise cls.skipException(
                'Share server migration tests are disabled.')
        utils.check_skip_if_microversion_not_supported('2.57')

    @classmethod
    def resource_setup(cls):
        super(MigrationShareServerBase, cls).resource_setup()
        cls.all_hosts = cls.shares_v2_client.list_pools(detail=True)
        cls.backends = set()
        for pool in cls.all_hosts['pools']:
            if pool['capabilities'].get('driver_handles_share_servers'):
                cls.backends.add(pool['name'].split('#')[0])

        if len(cls.backends) < 2:
            msg = ("Could not find the necessary backends. At least two"
                   " are needed to run the tests of share server migration")
            raise cls.skipException(msg)

        # create share type (generic)
        replication_type = CONF.share.backend_replication_type
        if replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=replication_type
            )
        extra_specs = {
            "replication_type": replication_type,
            "driver_handles_share_servers": CONF.share.multitenancy_enabled,
        }
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)

        # create two non routable IPs to be used in NFS access rulesi
        cls.access_rules_ip_rw = utils.rand_ip()
        cls.access_rules_ip_ro = utils.rand_ip()

    def _setup_migration(self, share):
        """Initial share server migration setup."""

        share = self.shares_v2_client.get_share(share['id'])['share']
        server_id = share['share_server_id']

        # (andrer) Verify if have at least one backend compatible with
        # the specified share server.
        dest_host, compatible = (
            self._choose_compatible_backend_for_share_server(server_id))

        snapshot = False
        if (compatible['supported_capabilities']['preserve_snapshots'] and
                share['snapshot_support']):
            snapshot = self.create_snapshot_wait_for_active(
                share['id'], cleanup_in_class=False)['id']

        # (andrer) Check the share export locations.
        old_exports = self.shares_v2_client.list_share_export_locations(
            share['id'])['export_locations']
        self.assertNotEmpty(old_exports)
        old_exports = [x['path'] for x in old_exports
                       if x['is_admin_only'] is False]
        self.assertNotEmpty(old_exports)

        # (andrer) Create the access rules, considering NFS and CIFS
        # protocols.
        access_rules = self._get_access_rule_data_for_protocols()
        for rule in access_rules:
            self.allow_access(
                share['id'], access_type=rule.get('access_type'),
                access_to=rule.get('access_to'),
                access_level=rule.get('access_level')
            )

        share = self.shares_v2_client.get_share(share['id'])['share']

        return share, server_id, dest_host, snapshot

    def _validate_state_of_resources(self, share, expected_status,
                                     snapshot_id):
        """Validates the share and snapshot status."""
        statuses = ((expected_status,)
                    if not isinstance(expected_status, (tuple, list, set))
                    else expected_status)

        share = self.shares_v2_client.get_share(share['id'])['share']
        self.assertIn(share['status'], statuses)

        if snapshot_id:
            snapshot = self.shares_v2_client.get_snapshot(
                snapshot_id)['snapshot']
            self.assertIn(snapshot['status'], statuses)

    def _validate_share_server_migration_complete(
            self, share, dest_host, dest_server_id, snapshot_id=None,
            share_network_id=None, version=CONF.share.max_api_microversion):
        """Validates the share server migration complete. """

        # Check the export locations
        new_exports = self.shares_v2_client.list_share_export_locations(
            share['id'], version=version)['export_locations']
        self.assertNotEmpty(new_exports)
        new_exports = [x['path'] for x in new_exports if
                       x['is_admin_only'] is False]
        self.assertNotEmpty(new_exports)

        # Check the share host, share_network, share_server and status.
        share = self.shares_v2_client.get_share(share['id'])['share']
        self.assertEqual(share['host'].split('#')[0], dest_host)
        self.assertEqual(share_network_id, share['share_network_id'])
        self.assertEqual(dest_server_id, share['share_server_id'])
        self.assertEqual(share['status'], constants.STATUS_AVAILABLE)

        # Check the snapshot status if possible.
        if snapshot_id:
            waiters.wait_for_resource_status(
                self.shares_v2_client, snapshot_id, constants.STATUS_AVAILABLE,
                resource_name='snapshot'
            )

        # Check the share server destination status.
        dest_server = self.shares_v2_client.show_share_server(
            dest_server_id)['share_server']
        self.assertIn(dest_server['task_state'],
                      constants.TASK_STATE_MIGRATION_SUCCESS)

        # Check if the access rules are in the share.
        rules = self.shares_v2_client.list_access_rules(
            share['id'])['access_list']
        if self.protocol == 'cifs':
            expected_rules = [{
                'state': constants.RULE_STATE_ACTIVE,
                'access_to': CONF.share.username_for_user_rules,
                'access_type': 'user',
                'access_level': 'rw',
            }]
        elif self.protocol == 'nfs':
            expected_rules = [{
                'state': constants.RULE_STATE_ACTIVE,
                'access_to': self.access_rules_ip_rw,
                'access_type': 'ip',
                'access_level': 'rw',
            }, {
                'state': constants.RULE_STATE_ACTIVE,
                'access_to': self.access_rules_ip_ro,
                'access_type': 'ip',
                'access_level': 'ro',
            }]

        filtered_rules = [{'state': rule['state'],
                           'access_to': rule['access_to'],
                           'access_level': rule['access_level'],
                           'access_type': rule['access_type']}
                          for rule in rules]

        for r in expected_rules:
            self.assertIn(r, filtered_rules)
        self.assertEqual(len(expected_rules), len(filtered_rules))

    @classmethod
    def _choose_compatible_backend_for_share_server(self, server_id):
        """Choose a compatible host for the share server migration."""
        for backend in self.backends:
            # This try is necessary since if you try migrate the share server
            # using the same backend and share network will raise an exception.
            try:
                compatibility = (
                    self.admin_shares_v2_client.share_server_migration_check(
                        share_server_id=server_id, host=backend))
            except exceptions.Conflict or exceptions.ServerFault:
                continue
            if compatibility['compatible']:
                return backend, compatibility

        raise self.skipException(
            "Not found compatible host for the share server migration.")

    def _choose_incompatible_backend_for_share_server(self, server_id):
        """Choose a not compatible host for the share server migration."""
        for backend in self.backends:
            # This try is necessary since if you try migrate the share server
            # using the same backend and share network will raise an exception.
            try:
                compatibility = (
                    self.admin_shares_v2_client.share_server_migration_check(
                        share_server_id=server_id, host=backend))
            except exceptions.Conflict or exceptions.ServerFault:
                continue
            if not compatibility['compatible']:
                return backend, compatibility

        raise self.skipException(
            "None of the hosts available are incompatible to perform a"
            " negative share server migration test.")

    def _get_share_server_destination_for_migration(self, src_server_id):
        """Find the destination share server chosen for the migration."""
        params = {'source_share_server_id': src_server_id,
                  'status': constants.STATUS_SERVER_MIGRATING_TO}
        dest_server = self.admin_shares_v2_client.list_share_servers(
            search_opts=params)['share_servers']
        dest_server_id = dest_server[0]['id'] if dest_server else None

        return dest_server_id

    def _get_access_rule_data_for_protocols(self):
        """Return fake data for access rules based on configured protocol."""
        if self.protocol == 'nfs':
            return [{
                'access_type': 'ip',
                'access_to': self.access_rules_ip_rw,
                'access_level': 'rw',
            }, {
                'access_type': 'ip',
                'access_to': self.access_rules_ip_ro,
                'access_level': 'ro',
            }]
        elif self.protocol == 'cifs':
            return [{
                'access_type': 'user',
                'access_to': CONF.share.username_for_user_rules,
                'access_level': 'rw',
            }]
        else:
            message = "Unrecognized protocol and access rules configuration"
            raise self.skipException(message)


@ddt.ddt
class ShareServerMigrationBasicNFS(MigrationShareServerBase):
    protocol = "nfs"

    @decorators.idempotent_id('5b84bcb6-17d8-4073-8e02-53b54aee6f8b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_share_server_migration_cancel(self):
        """Test the share server migration cancel."""
        share_network_id = self.provide_share_network(
            self.shares_v2_client, self.networks_client)
        share = self.create_share(share_protocol=self.protocol,
                                  share_type_id=self.share_type['id'],
                                  share_network_id=share_network_id,
                                  cleanup_in_class=False)
        share = self.shares_v2_client.get_share(share['id'])['share']

        # Initial migration setup.
        share, src_server_id, dest_host, snapshot_id = self._setup_migration(
            share)

        preserve_snapshots = True if snapshot_id else False

        # Start share server migration.
        self.shares_v2_client.share_server_migration_start(
            src_server_id, dest_host, preserve_snapshots=preserve_snapshots)

        expected_state = constants.TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE
        timeout = CONF.share.share_server_migration_timeout
        waiters.wait_for_resource_status(
            self.shares_v2_client, src_server_id,
            expected_state, resource_name='share_server',
            status_attr='task_state', timeout=timeout
        )

        # Get for the destination share server.
        dest_server_id = self._get_share_server_destination_for_migration(
            src_server_id)

        dest_server = self.shares_v2_client.show_share_server(
            dest_server_id)['share_server']
        self.assertEqual(dest_host, dest_server['host'])
        self.assertEqual(share_network_id, dest_server['share_network_id'])

        # Validate the share instances status.
        share_status = constants.STATUS_SERVER_MIGRATING
        self._validate_state_of_resources(share, share_status, snapshot_id)

        # Cancel the share server migration.
        self.shares_v2_client.share_server_migration_cancel(src_server_id)

        # Wait for the migration cancelled status.
        expected_state = constants.TASK_STATE_MIGRATION_CANCELLED
        waiters.wait_for_resource_status(
            self.shares_v2_client, src_server_id,
            expected_state, resource_name='share_server',
            status_attr='task_state')

        # After the cancel operation, we need to validate again the resources.
        expected_status = constants.STATUS_AVAILABLE
        self._validate_state_of_resources(share, expected_status, snapshot_id)

    @decorators.idempotent_id('99e439a8-a716-4205-bf5b-af50128cb908')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(
        (False, False),
        (True, False),
    )
    @ddt.unpack
    def test_share_server_migration_complete(
        self, new_share_network, check_with_replica
    ):
        self._test_share_server_migration_complete(
            new_share_network, check_with_replica)

    @decorators.idempotent_id('ae0e9e6c-3a77-4c4b-907b-8a793f88c734')
    @testtools.skipUnless(CONF.share.run_positive_migration_replica_tests,
                          'Share server migration with replica test '
                          'is disabled.')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(
        (True, True)
    )
    @ddt.unpack
    def test_share_server_migration_complete_allow_replica(
        self, new_share_network, check_with_replica
    ):
        self._test_share_server_migration_complete(
            new_share_network, check_with_replica)

    def _test_share_server_migration_complete(self, new_share_network,
                                              check_with_replica):
        """Test the share server migration complete."""
        share_network_id = self.provide_share_network(
            self.shares_v2_client, self.networks_client)
        dest_share_network_id = share_network_id
        if new_share_network:
            src_share_network = self.shares_v2_client.get_share_network(
                share_network_id)['share_network']
            share_net_info = (
                utils.share_network_get_default_subnet(src_share_network))
            dest_share_network_id = self.create_share_network(
                neutron_net_id=share_net_info['neutron_net_id'],
                neutron_subnet_id=share_net_info['neutron_subnet_id'],
                cleanup_in_class=False)['id']

        share = self.create_share(share_protocol=self.protocol,
                                  share_type_id=self.share_type['id'],
                                  share_network_id=share_network_id,
                                  cleanup_in_class=False)
        share = self.shares_v2_client.get_share(share['id'])['share']

        # Initial migration setup.
        share, src_server_id, dest_host, snapshot_id = self._setup_migration(
            share)

        preserve_snapshots = True if snapshot_id else False

        replica = {}
        if check_with_replica:
            replica = self.create_share_replica(
                share['id'],
                cleanup_in_class=False)

        # Start share server migration.
        self.shares_v2_client.share_server_migration_start(
            src_server_id, dest_host,
            new_share_network_id=dest_share_network_id,
            preserve_snapshots=preserve_snapshots)

        expected_state = constants.TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE
        timeout = CONF.share.share_server_migration_timeout
        waiters.wait_for_resource_status(
            self.shares_v2_client, src_server_id,
            expected_state, resource_name='share_server',
            status_attr='task_state', timeout=timeout
        )
        # Get for the destination share server.
        dest_server_id = self._get_share_server_destination_for_migration(
            src_server_id)

        dest_server = self.shares_v2_client.show_share_server(
            dest_server_id)['share_server']
        self.assertEqual(dest_host, dest_server['host'])
        self.assertEqual(dest_share_network_id,
                         dest_server['share_network_id'])

        expected_status = constants.STATUS_SERVER_MIGRATING
        self._validate_state_of_resources(share, expected_status, snapshot_id)

        # Share server migration complete.
        self.shares_v2_client.share_server_migration_complete(src_server_id)

        # It's necessary wait for the destination server went to active status.
        expected_status = constants.SERVER_STATE_ACTIVE
        waiters.wait_for_resource_status(
            self.shares_v2_client, dest_server_id, expected_status,
            resource_name='share_server'
        )

        # Validate the share server migration complete.
        share = self.shares_v2_client.get_share(share['id'])['share']
        self._validate_share_server_migration_complete(
            share, dest_host, dest_server_id, snapshot_id=snapshot_id,
            share_network_id=dest_share_network_id)

        # Source share server is only deleted after Wallaby release (2.63).
        if utils.is_microversion_gt(CONF.share.max_api_microversion, "2.63"):
            self.admin_shares_client.wait_for_resource_deletion(
                server_id=src_server_id)

        if check_with_replica:
            replica = self.shares_v2_client.get_share_replica(
                replica["id"])['share_replica']
            self.assertEqual(constants.REPLICATION_STATE_IN_SYNC,
                             replica["replica_state"])

    @decorators.idempotent_id('52e154eb-2d39-45af-b5c1-49ea569ab804')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(True, False)
    def test_share_server_migration_check(self, compatible):
        """The share server migration check compatibility tests."""
        share = self.create_share(share_protocol=self.protocol,
                                  share_type_id=self.share_type['id'],
                                  cleanup_in_class=False)
        share = self.shares_v2_client.get_share(share['id'])['share']
        # Find a backend compatible or not for the share server
        # check compatibility operation.
        if compatible:
            dest_host, result = (
                self._choose_compatible_backend_for_share_server(
                    server_id=share['share_server_id']))
            self.assertTrue(result['compatible'])
            self.assertEqual(result['requested_capabilities']['host'],
                             dest_host)
        else:
            dest_host, result = (
                self._choose_incompatible_backend_for_share_server(
                    server_id=share['share_server_id']))
            self.assertFalse(result['compatible'])
            self.assertEqual(result['requested_capabilities'].get('host'),
                             dest_host)


class ShareServerMigrationBasicCIFS(ShareServerMigrationBasicNFS):
    protocol = "cifs"
